package net.wanji.utc.config.security;

import net.wanji.common.framework.rest.HeaderMapRequestWrapper;
import net.wanji.common.utils.cookie.CookieUtil;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.common.utils.cache.IgniteClientCacheUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpHeaders;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableResourceServer;
import org.springframework.security.web.context.SecurityContextPersistenceFilter;
import org.springframework.security.web.util.matcher.RequestMatcher;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

import static net.wanji.common.framework.Constants.Cache.LOGIN_CACHE;

/**
 * @Description: 资源服务器配置
 * @Author
 * @Date 2020/4/12 21:01
 */
@Configuration
@EnableResourceServer
@EnableGlobalMethodSecurity(prePostEnabled = true)
@Slf4j
public class ResourceServerConfig extends IdentityResourceServerConfig {
    private static  String profileActive;
    @Value("${spring.profiles.active}")
    public void setProfileActive(String profileActive) {
        ResourceServerConfig.profileActive = profileActive;
    }

    @Override
    public void configure(HttpSecurity http) throws Exception {
//        放开全部
        http.authorizeRequests()
                .antMatchers(whiteList.toArray(new String[whiteList.size()])).permitAll()
                .requestMatchers(new IpRequestMatcher()).permitAll()
                .anyRequest().authenticated()
                .and().cors()
                .and().csrf().disable()
                .addFilterBefore(new OncePerRequestFilter() {
                    @Override
                    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
                        String breakerToken = " ";
                        // 判断header里有没有authorization，有则赋值给breakertoken，没有则根据tokenid
                        if (StringUtils.isNotBlank(request.getHeader(HttpHeaders.AUTHORIZATION))) {
                            breakerToken = request.getHeader(HttpHeaders.AUTHORIZATION);
                        } else {
                            String tokenId = "";
                            // 移动端的tokenid在请求头里，pc端的在cookie里
                            String mobileTokenId = request.getHeader("tokenId");
                            if (StringUtils.isNotBlank(mobileTokenId)) {
                                tokenId = mobileTokenId;
                            } else {
                                tokenId = CookieUtil.getUid(request, "tokenId");
                            }
                            if (StringUtils.isNotBlank(tokenId)) {
                                try {

                                    OAuth2AccessToken oAuth2AccessToken = (OAuth2AccessToken) IgniteClientCacheUtil.getValue(LOGIN_CACHE, tokenId);
                                    breakerToken = oAuth2AccessToken.getTokenType() + " " + oAuth2AccessToken.getValue();
                                } catch (Exception e) {
                                    log.error("",e);
                                }
                            }
                        }
                        HeaderMapRequestWrapper requestWrapper = new HeaderMapRequestWrapper(request);
                        if (StringUtils.isNotBlank(breakerToken)) {
                            //将cookie中的token放到header，供oauth鉴权使用
                            requestWrapper.addHeader(HttpHeaders.AUTHORIZATION, breakerToken);
                        }
                        filterChain.doFilter(requestWrapper, response);
                    }
                }, SecurityContextPersistenceFilter.class);
    }

    //内部类，IP白名单匹配器
    private static class IpRequestMatcher implements RequestMatcher{
        //IPV4 和IPV6本地地址
        private static final List<String> ALLOWED_IPS = Arrays.asList("127.0.0.1","173.17.0.1","0:0:0:0:0:0:0:1");

        @Override
        public boolean matches(HttpServletRequest request) {
            String remoteAddr = request.getRemoteAddr();
            return ALLOWED_IPS.contains(remoteAddr);
        }

        @Override
        public MatchResult matcher(HttpServletRequest request) {
            return RequestMatcher.super.matcher(request);
        }
    }

    //访问白名单
    private static final List<String> whiteList = new ArrayList<String>() {{
        //swagger
        add("/webjars/**");
        add("/resources/**");
        add("/swagger-ui.html");
        add("/swagger-resources/**");
        add("/v2/api-docs");
        add("/getSwaggerToken");
        add("/");
        add("/**/*.html");
        add("/**/*.htm");
        add("/**/*.woff");
        add("/**/*.woff2");
        add("/**/*.ttf");
        add("/**/*.svg");
        add("/**/*.css");
        add("/**/*.png");
        add("/**/*.jpg");
        add("/**/*.gif");
        add("/**/*.ico");
        add("/**/*.json");
        add("/**/*.js");
        add("/**/*.txt");
        add("/utc/signalStatus/**");
        //add("/cross/*");
       // add("/**") ;
//        add("/**");
        if (Objects.equals("test",profileActive)) {
            //测试环境放开接口
            add("/**");
        }
    }};
}
